﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using Microsoft.Win32;
using System.IO;
using System.Net.NetworkInformation; //for ping
using System.Net;

namespace ProxySwap
{
    public partial class ProxySwap_Window : Form
    {
        #region Variables
        string XMLPath = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + @"\ProxySwap";
        string XMLFile = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + @"\ProxySwap\ProxySwap.xml";//@"C:\Users\Default\AppData\Roaming\ProxySwap\ProxySwap.xml";
        string ProxyTestResult = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + @"\ProxySwap\ProxyTestResult.xml";
        ListViewItem lvItem;
        public string newComment;
        public string pulledComment;
        public string rIP, rRemoteIP, rDomain;              //--- for details dialog ---//
        public string rPort, rIsUsingProxy, rClientIPVisible;  //--- for details dialog ---//
        #endregion

        public ProxySwap_Window()
        {
            InitializeComponent();
            GeometryFromString(Properties.Settings.Default.WindowGeometry, this); 

            Get_Proxies();
            GetProxyFromRegistry();
        }

        #region Saving And Restoring Size, Location and State
        /// <summary>
        /// Gotten code from http://www.codeproject.com/Articles/25510/Restore-Form-Position-and-Size-in-C
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ProxySwap_Window_FormClosing(object sender, FormClosingEventArgs e)
        {
            Properties.Settings.Default.WindowGeometry = GeometryToString(this);
            Properties.Settings.Default.Save();
        }

        public static void GeometryFromString(string thisWindowGeometry, Form formIn)
        {
            if (string.IsNullOrEmpty(thisWindowGeometry) == true)
            {
                return;
            }
            string[] numbers = thisWindowGeometry.Split('|');
            string windowString = numbers[4];
            if (windowString == "Normal")
            {
                Point windowPoint = new Point(int.Parse(numbers[0]),
                    int.Parse(numbers[1]));
                Size windowSize = new Size(int.Parse(numbers[2]),
                    int.Parse(numbers[3]));

                bool locOkay = GeometryIsBizarreLocation(windowPoint, windowSize);
                bool sizeOkay = GeometryIsBizarreSize(windowSize);

                if (locOkay == true && sizeOkay == true)
                {
                    formIn.Location = windowPoint;
                    formIn.Size = windowSize;
                    formIn.StartPosition = FormStartPosition.Manual;
                    formIn.WindowState = FormWindowState.Normal;
                }
                else if (sizeOkay == true)
                {
                    formIn.Size = windowSize;
                }
            }
            else if (windowString == "Maximized")
            {
                formIn.Location = new Point(100, 100);
                formIn.StartPosition = FormStartPosition.Manual;
                formIn.WindowState = FormWindowState.Maximized;
            }
        }

        private static bool GeometryIsBizarreLocation(Point loc, Size size)
        {
            bool locOkay;
            if (loc.X < 0 || loc.Y < 0)
            {
                locOkay = false;
            }
            else if (loc.X + size.Width > Screen.PrimaryScreen.WorkingArea.Width)
            {
                locOkay = false;
            }
            else if (loc.Y + size.Height > Screen.PrimaryScreen.WorkingArea.Height)
            {
                locOkay = false;
            }
            else
            {
                locOkay = true;
            }
            return locOkay;
        }

        private static bool GeometryIsBizarreSize(Size size)
        {
            return (size.Height <= Screen.PrimaryScreen.WorkingArea.Height &&
                size.Width <= Screen.PrimaryScreen.WorkingArea.Width);
        }

        public static string GeometryToString(Form mainForm)
        {
            return mainForm.Location.X.ToString() + "|" +
                mainForm.Location.Y.ToString() + "|" +
                mainForm.Size.Width.ToString() + "|" +
                mainForm.Size.Height.ToString() + "|" +
                mainForm.WindowState.ToString();
        }
        #endregion

        #region Create XML File
        private void CreateXMLFile()
        {
            Directory.CreateDirectory(XMLPath);
            FileStream fs = File.Create(XMLFile);
            fs.Dispose();
        }
        #endregion

        #region Create ProxyTestResult File
        private void CreateProxyTestResultFile()
        {
            Directory.CreateDirectory(XMLPath);
            FileStream fs = File.Create(ProxyTestResult);
            fs.Dispose();
        }
        #endregion

        #region Get Proxy From Registry
        private void GetProxyFromRegistry()
        {
            RegistryKey SystemProxy = Registry.CurrentUser.CreateSubKey(@"Software\Microsoft\Windows\CurrentVersion\Internet Settings");
            try
            {
                string UsedProxy = SystemProxy.GetValue("ProxyServer").ToString();
                foreach (ListViewItem item in listView1.Items)
                {
                    if (UsedProxy == item.Text + ":" + item.SubItems[1].Text)
                    {
                        item.Checked = true;
                    }
                    else
                    {
                        item.Checked = false;
                    }
                }
            }
            catch (Exception) 
            {
            }
        }
        #endregion

        #region Get Proxies From XML To ListView
        private void Get_Proxies()
        {
            listView1.Items.Clear();
            XmlDocument ProxyList = new System.Xml.XmlDocument();
            try
            {
                ProxyList.Load(XMLFile);
            }
            catch (Exception) 
            {
            }

            foreach (XmlNode IPNode in ProxyList.SelectNodes("/ProxyList/IP"))
            {
                lvItem = listView1.Items.Add(IPNode.InnerText);
                lvItem.SubItems.Add(new ListViewItem.ListViewSubItem(lvItem, IPNode.Attributes["port"].InnerText)); ;
                lvItem.SubItems.Add(new ListViewItem.ListViewSubItem(lvItem, IPNode.Attributes["country"].InnerText)); ;
                lvItem.SubItems.Add(""); ;  // --- delay  --- // --- needed for creating a subitem so I could change it's text by index in Pinging Proxy
                lvItem.SubItems.Add(""); ;  // --- status --- //
                lvItem.SubItems.Add(new ListViewItem.ListViewSubItem(lvItem, IPNode.Attributes["comment"].InnerText)); ;

            }            
        }
        #endregion

        #region Add Proxies To ListView
        private void AddProxiesToListView()
        {
            if (string.IsNullOrEmpty(Proxy_textBox.Text) || string.IsNullOrEmpty(Port_textBox.Text))
            {
                MessageBox.Show("Proxy and/or Port fields cannot be left empty!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                ListViewItem lvItem = new ListViewItem();
                lvItem.Text = Proxy_textBox.Text;
                lvItem.SubItems.Add(Port_textBox.Text);
                lvItem.SubItems.Add(Country_textBox.Text);
                lvItem.SubItems.Add("");   // --- delay  --- // --- required for subitems being created, so i can change their text by index in Pinging Proxy region
                lvItem.SubItems.Add("");   // --- status --- //
                lvItem.SubItems.Add(Comment_textBox.Text);
                listView1.Items.Add(lvItem);

                Proxy_textBox.Text = "";
                Port_textBox.Text = "";
                Country_textBox.Text = "";
                Comment_textBox.Text = "";
            }

            ///////
            
        }
#endregion

        #region Remove Individual Proxies From ListView
        private void DeleteItem()
        {
            foreach (ListViewItem item in listView1.SelectedItems)
            {
                item.Remove();
                WriteListViewToXML();
            }
        }
        #endregion

        #region Write ListView Items To XML File
        private void WriteListViewToXML()
        {
            XmlTextWriter writer = new System.Xml.XmlTextWriter(XMLFile, null);

            writer.WriteStartElement("ProxyList");

            for (int i = 0; i < listView1.Items.Count; i++)
            {
                writer.WriteStartElement("IP");
                writer.WriteAttributeString("port", listView1.Items[i].SubItems[1].Text); ;
                writer.WriteAttributeString("country", listView1.Items[i].SubItems[2].Text); ;
                writer.WriteAttributeString("comment", listView1.Items[i].SubItems[5].Text); ;
                writer.WriteValue(listView1.Items[i].SubItems[0].Text);
                writer.WriteEndElement();
            }
            writer.WriteEndElement();
            writer.Close();

        }
        #endregion

        #region Allowing To Check Only One Item In ListView
        void listView1_ItemCheck(object sender, ItemCheckEventArgs e)
        {
            if (e.CurrentValue == CheckState.Checked)
                return;
            foreach (ListViewItem item in listView1.Items)
            {
                if (item.Index != e.Index)
                {
                    item.Checked = false;
                }
            }
        }

        #endregion

        #region Pinging Proxy
        private void Ping_Current_Proxy() //--- pinging checeked proxy on pressing Apply button ---//
        {
            foreach (ListViewItem item in listView1.CheckedItems)
            {
                try
                {
                    Ping pingIP = new Ping();
                    PingReply pingReply = pingIP.Send(item.Text);
                    if (pingReply.Status == IPStatus.Success)
                    {
                        item.SubItems[3].Text = pingReply.RoundtripTime.ToString();
                        item.SubItems[4].Text = pingReply.Status.ToString();
                        item.UseItemStyleForSubItems = false;
                        item.SubItems[4].ForeColor = Color.Green;
                    }
                    else
                    {
                        item.SubItems[3].Text = "";
                        item.SubItems[4].Text = pingReply.Status.ToString();
                        item.UseItemStyleForSubItems = false;
                        item.SubItems[4].ForeColor = Color.Red;
                    }
                }
                catch (Exception)
                {
                    MessageBox.Show("Could not ping remote computer! \nMake sure you have internet connection and you've typed IP address correctly! \n[XXX.XXX.XXX.XXX]", "Error pinging selected IP", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    item.SubItems[3].Text = "";
                    item.SubItems[4].Text = "Failed";
                    item.UseItemStyleForSubItems = false;
                    item.SubItems[4].ForeColor = Color.Red;
                }
            }
        }

        private void Ping_Selected_Proxy() //--- pinging selected proxy via context menu ---//
        {
            foreach (ListViewItem item in listView1.SelectedItems)
            {
                try
                {
                    Ping pingIP = new Ping();
                    PingReply pingReply = pingIP.Send(item.Text);
                    if (pingReply.Status == IPStatus.Success)
                    {
                        item.SubItems[3].Text = pingReply.RoundtripTime.ToString();
                        item.SubItems[4].Text = pingReply.Status.ToString();
                        item.UseItemStyleForSubItems = false;
                        item.SubItems[4].ForeColor = Color.Green;
                    }
                    else
                    {
                        item.SubItems[3].Text = "";
                        item.SubItems[4].Text = pingReply.Status.ToString();
                        item.UseItemStyleForSubItems = false;
                        item.SubItems[4].ForeColor = Color.Red;
                    }
                }
                catch (Exception)
                {
                    MessageBox.Show("Could not ping remote computer! \nMake sure you have internet connection and you've typed IP address correctly! \n[XXX.XXX.XXX.XXX]", "Error pinging selected IP", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    item.SubItems[3].Text = "";
                    item.SubItems[4].Text = "Failed";
                    item.UseItemStyleForSubItems = false;
                    item.SubItems[4].ForeColor = Color.Red;
                }
            }
        }

        private void Ping_All_Proxies()
        {
            foreach (ListViewItem item in listView1.Items)
            {
                try
                {
                    Ping pingIP = new Ping();
                    PingReply pingReply = pingIP.Send(item.Text);
                    if (pingReply.Status == IPStatus.Success)
                    {
                        item.SubItems[3].Text = pingReply.RoundtripTime.ToString();
                        item.SubItems[4].Text = pingReply.Status.ToString();
                        item.UseItemStyleForSubItems = false;
                        item.SubItems[4].ForeColor = Color.Green;
                    }
                    else
                    {
                        item.SubItems[3].Text = "";
                        item.SubItems[4].Text = pingReply.Status.ToString();
                        item.UseItemStyleForSubItems = false;
                        item.SubItems[4].ForeColor = Color.Red;
                    }
                }
                catch (Exception)
                {
                    MessageBox.Show("Could not ping \"" + item.Text + "\"\nMake sure you've typed IP address correctly! \n[XXX.XXX.XXX.XXX]", "Error pinging " + item.Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    item.SubItems[3].Text = "";
                    item.SubItems[4].Text = "Failed";
                    item.UseItemStyleForSubItems = false;
                    item.SubItems[4].ForeColor = Color.Red;
                }
            }
        }
        #endregion

        #region Full Proxy Test
        private void FullProxyTest()
        {
            CreateProxyTestResultFile();
            foreach (ListViewItem item in listView1.SelectedItems)
            {
                HttpWebRequest request = WebRequest.Create("http://www.zahodi-ka.ru/wm/proxy-checker/check.fcgi?p=" + "http://" + item.SubItems[0].Text + ":" + item.SubItems[1].Text + "&timeout=20") as HttpWebRequest;

                using (StreamWriter file = new StreamWriter(ProxyTestResult))
                {
                    using (HttpWebResponse response = request.GetResponse() as HttpWebResponse)
                    {
                        StreamReader reader = new StreamReader(response.GetResponseStream());
                        file.WriteLine(reader.ReadToEnd());
                    }
                }
            }
        }
        #endregion

        #region Parse XML Test Result File
        private void ParseXMLResultFile()
        {
            XmlDocument xmlResults = new XmlDocument();
            try
            {
                xmlResults.Load(ProxyTestResult);
            }
            catch (Exception)
            {
            }

            string root = xmlResults.SelectSingleNode("/*").Name;

            if (root.ToString() == "er")
            {
                foreach (XmlNode resultNode in xmlResults.SelectNodes("/er"))
                {
                    if (resultNode.ChildNodes[1].InnerText == "001")
                    {
                        MessageBox.Show("Invalid proxy server format or domain.\nPlease, make sure you've entered correct information.", "Invalid proxy server", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                    if (resultNode.ChildNodes[1].InnerText == "002")
                    {
                        MessageBox.Show("Unable to retrieve IP Address for current proxy server.\nDestination host is unreachable.", "Destination host is unreachable", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                    else
                    {
                        MessageBox.Show("API returned an unknown error.", "Unknown error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
            else
            {
                if (root.ToString() == "ok")
                {
                    foreach (XmlNode resultNode in xmlResults.SelectNodes("/ok"))
                    {
                        rIP = resultNode.ChildNodes[0].InnerText;
                        rPort = resultNode.ChildNodes[6].InnerText;
                        rRemoteIP = resultNode.ChildNodes[1].InnerText;
                        rClientIPVisible = resultNode.ChildNodes[2].InnerText;
                        rIsUsingProxy = resultNode.ChildNodes[3].InnerText;
                        rDomain = resultNode.ChildNodes[4].InnerText;

                        dlgDetails dialog = new dlgDetails(rIP, rPort, rRemoteIP, rClientIPVisible, rIsUsingProxy, rDomain);
                        dialog.ShowDialog();
                        continue;
                    }
                }
                else
                {
                    MessageBox.Show("Could not recognize respond from API.\nPlease, see C:\\Users\\%username%\\AppData\\Roaming\\ProxySwap\\ProxyTestResult.xml for details.", "Unknown error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
        #endregion

        #region Apply Proxy In Registry
        private void ApplyChanges()
        {
            ListView.CheckedListViewItemCollection checkedItem = listView1.CheckedItems;
            RegistryKey SystemProxy = Registry.CurrentUser.CreateSubKey(@"Software\Microsoft\Windows\CurrentVersion\Internet Settings");

            try
            {
                if (listView1.CheckedItems.Count > 0)
                {
                    foreach (ListViewItem item in listView1.CheckedItems)
                    {
                        string ProxyPort = item.SubItems[0].Text + ":" + item.SubItems[1].Text;
                        SystemProxy.SetValue("ProxyServer", ProxyPort);
                        SystemProxy.SetValue("ProxyEnable", "1", RegistryValueKind.DWord);
                    }
                }
                else
                {
                    try
                    {
                        SystemProxy.SetValue("ProxyEnable", "0", RegistryValueKind.DWord);
                        SystemProxy.SetValue("ProxyServer", "");
                    }
                    catch (Exception exc)
                    {
                        MessageBox.Show(exc.Message);
                    }
                }
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message);
            }
        }
        #endregion

        #region Disabling Proxy
        private void DisableSystemProxy()
        {
            RegistryKey SystemProxy = Registry.CurrentUser.CreateSubKey(@"Software\Microsoft\Windows\CurrentVersion\Internet Settings");

            try
            {
                SystemProxy.SetValue("ProxyServer", "");
                SystemProxy.SetValue("ProxyEnable", "0", RegistryValueKind.DWord);
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message);
            }

            foreach (ListViewItem item in listView1.Items)
            {
                item.Checked = false;
            }
        }
        #endregion

        #region Window Buttons Behavior
        private void Add_button_Click(object sender, EventArgs e)
        {
            AddProxiesToListView();
            CreateXMLFile();
            WriteListViewToXML();
        }

        private void Help_button_Click(object sender, EventArgs e)
        {
            dlgAbout about = new dlgAbout();
            about.ShowDialog();
            //MessageBox.Show("This application helps you to change system proxy easily!\n\nartman_12@hotmail.com \n2012", "Help", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void Test_Proxy_button_Click(object sender, EventArgs e)
        {
            Ping_All_Proxies();
        }

        private void Clear_button_Click(object sender, EventArgs e)
        {
            DisableSystemProxy();
        }

        private void Apply_button_Click(object sender, EventArgs e)
        {
            ApplyChanges();
            Ping_Current_Proxy();
        }
        #endregion

        #region Context Menu
        private void contextMenuStrip1_Opening(object sender, CancelEventArgs e)
        {
            if (listView1.SelectedItems.Count > 1)
            {
                getProxyDetailsToolStripMenuItem.Enabled = false;
                editCommentToolStripMenuItem.Enabled = false;
            }
        }

        private void pingToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Ping_Selected_Proxy();
        }

        private void editCommentToolStripMenuItem_Click(object sender, EventArgs e)
        {
            foreach (ListViewItem item in listView1.SelectedItems)
            {
                pulledComment = item.SubItems[5].Text;
                using (dlgEditComment dialog = new dlgEditComment(pulledComment))
                {
                    if (dialog.ShowDialog(this) == DialogResult.OK)
                    {
                        item.SubItems[5].Text = dialog.newComment;
                        pulledComment = item.SubItems[5].Text;
                        CreateXMLFile();
                        WriteListViewToXML();
                    }
                }
            }
        }

        private void deleteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DeleteItem();
        }

        private void fullProxyTestToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FullProxyTest();
            ParseXMLResultFile();
        }
        #endregion
    }
}
